﻿using System;
using System.ComponentModel;
using ResourceTest.Services;

namespace ResourceTest.Helpers
{
    /// <summary>
    /// Helper class for binding to resource strings
    /// </summary>
    public class LocalisationHelper : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        protected void RaisePropertyChanged(string propertyName)
        {
            var evt = PropertyChanged;

            if (evt != null)
                evt.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Initializes a new instance of the LocalisationHelper class.
        /// </summary>
        public LocalisationHelper()
        {
            if (!DesignHelpers.IsInDesignModeStatic)
            {
                // Refresh all the bindings when the locale changes
                ResourceManagerService.LocaleChanged += (s, e) =>
                    {
                        RaisePropertyChanged(string.Empty);
                    };
            }
        }

        /// <summary>
        /// Gets a resource string from the ResourceManager
        /// 
        /// You can bind to this property using the .[KEY] syntax e.g.:
        /// 
        /// {Binding Source={StaticResource localisation}, Path=.[MainScreenResources.IntroTextLine1]}
        /// </summary>
        /// <param name="Key">Key to retrieve in the format [ManagerName].[ResourceKey]</param>
        /// <returns></returns>
        public string this[string Key]
        {
            get
            {
                if (!validKey(Key))
                    throw new ArgumentException(@"Key is not in the valid [ManagerName].[ResourceKey] format");

                if (DesignHelpers.IsInDesignModeStatic)
                    throw new Exception("Design mode is not supported");

                return ResourceManagerService.GetResourceString(GetManagerKey(Key), GetResourceKey(Key));
            }
        }

        #region Private Key Methods
        private bool validKey(string input)
        {
            return input.Contains(".");
        }

        private string GetManagerKey(string input)
        {
            return input.Split('.')[0];
        }

        private string GetResourceKey(string input)
        {
            return input.Substring(input.IndexOf('.') + 1);
        }
        #endregion
    }
}
